import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.codehaus.jackson.map.ObjectMapper;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.WorkingMemoryEntryPoint;
import org.junit.Before;
import org.junit.Test;

public class TestMemoryProblem {
	private StatefulKnowledgeSession kSession = null;
	private static SimpleDateFormat formatTimeComMS = new SimpleDateFormat("HH:mm:ss.SSS");
	private ObjectMapper mapper;

	@Before
	public void setUpTest() {
		KnowledgeBuilder kBuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();

		URL url = TestMemoryProblem.class.getClassLoader().getResource("memory_problem.drl");
		kBuilder.add(ResourceFactory.newFileResource(url.getPath()), ResourceType.DRL);
		if (kBuilder.hasErrors()) {
			throw new IllegalStateException("Error loading the rules: " + kBuilder.getErrors().toString());
		}

		KnowledgeBaseConfiguration configBase = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
		configBase.setOption(EventProcessingOption.STREAM);

		KnowledgeBase kBase = KnowledgeBaseFactory.newKnowledgeBase(configBase);
		kBase.addKnowledgePackages(kBuilder.getKnowledgePackages());

		kSession = kBase.newStatefulKnowledgeSession();
		mapper = new ObjectMapper();
		kSession.setGlobal("jsonConverter", mapper);
	}

	private void loadPOS() {
		WorkingMemoryEntryPoint entryPointConvertAndInsertPOS = kSession
				.getWorkingMemoryEntryPoint("convertAndInsertPOS");
		log("Loading POS");
		for (int i = 1; i <= 20000; i++) {
			if ((i % 1000) == 0) {
				log("Inserting POST " + i);
				kSession.fireAllRules();
			}
			entryPointConvertAndInsertPOS.insert(getPOSInJson("POS" + i));
		}
		kSession.fireAllRules();
		log("POS loaded");
	}

	private void loadCustomer() {
		WorkingMemoryEntryPoint entryPointConvertAndInsertCustomer = kSession
				.getWorkingMemoryEntryPoint("convertAndInsertCustomer");
		for (int i = 1; i <= 10000; i++) {
			if ((i % 1000) == 0) {
				log("Inserting customer " + i);
				kSession.fireAllRules();
			}
			entryPointConvertAndInsertCustomer.insert(getCustomerInJson(String.valueOf(i)));
		}
		kSession.fireAllRules();
		log("Customers loadaed");
	}

	@Test
	public void test() throws Exception {
		loadPOS();
		loadCustomer();
	}

	private String getPOSInJson(String logicNumber) {
		StringBuilder str = new StringBuilder();
		str.append("{\"logicNumber\":\"").append(logicNumber).append("\"}");
		return str.toString();
	}

	private String getCustomerInJson(String id) {
		StringBuilder str = new StringBuilder();
		str.append("{\"id\":\"").append(id).append("\"}");
		return str.toString();
	}

	public static void log(String msg) {
		System.out.println(formatTimeComMS.format(new Date()) + " - " + msg);
	}

}